@@ -3,15 +3,15 @@ from UserDict import DictMixin
 __author__ = 'leifj'
 
 import os
-import rsa_x509_pem
+import xmlsec.rsa_x509_pem as rsa_x509_pem
 import lxml.etree as etree
 import logging
 import base64
 import hashlib
 import copy
-import int_to_bytes as itb
+import xmlsec.int_to_bytes as itb
 from lxml.builder import ElementMaker
-from exceptions import XMLSigException
+from xmlsec.exceptions import XMLSigException
 import re
 import htmlentitydefs
 
@@ -43,6 +43,7 @@ ALGORITHM_SIGNATURE_RSA_SHA1 = "http://www.w3.org/2000/09/xmldsig#rsa-sha1"
 # and includes https://github.com/andrewdyates/rsa_x509_pem with
 # permission from the author.
 
+
 class CertDict(DictMixin):
     """
     Extract all X509Certificate XML elements and create a dict-like object
@@ -86,8 +87,8 @@ def _find_matching_cert(t, fp):
     """
     if t is None:
         return None
-    for hash, pem in CertDict(t).iteritems():
-        if fp.lower() == hash:
+    for cfp, pem in CertDict(t).iteritems():
+        if fp.lower() == cfp:
             return pem
     return None
 
@@ -100,6 +101,8 @@ def _load_keyspec(keyspec, private=False, signature_element=None):
     given specification. For example, if keyspec is a a PKCS#11 reference to a
     private key then naturally the key itself is not available.
 
+    :param private:
+    :param signature_element:
     Possible keyspecs, in evaluation order :
 
       - a callable.    Return a partial dict with 'f_private' set to the keyspec.
@@ -133,15 +136,14 @@ def _load_keyspec(keyspec, private=False, signature_element=None):
     if private and hasattr(keyspec, '__call__'):
         return {'keyspec': keyspec,
                 'source': 'callable',
-                'f_private': keyspec,
-        }
+                'f_private': keyspec}
     if isinstance(keyspec, basestring):
         if os.path.isfile(keyspec):
             with open(keyspec) as c:
                 data = c.read()
             source = 'file'
         elif private and keyspec.startswith("pkcs11://"):
-            import pk11
+            import xmlsec.pk11
 
             key_f_private, data = pk11.signer(keyspec)
             logging.debug("Using pkcs11 signing key: %s" % key_f_private)
@@ -207,7 +209,7 @@ def b64e(s):
     return s.encode('base64').replace('\n', '')
 
 
-def _signed_value(data, key_size, do_pad, hash_alg): # TODO Do proper asn1 CMS
+def _signed_value(data, key_size, do_pad, hash_alg):  # TODO Do proper asn1 CMS
     """Return unencrypted rsa-sha1 signature value `padded_digest` from `data`.
 
     The resulting signed value will be in the form:
@@ -279,7 +281,7 @@ def _remove_child_comments(t):
     return t
 
 
-def _process_references(t, sig=None):
+def _process_references(t, sig=None, return_verified=True):
     """
     :returns: hash algorithm as string
     """
@@ -287,27 +289,31 @@ def _process_references(t, sig=None):
         sig = t.find(".//{%s}Signature" % NS['ds'])
     hash_alg = None
     for ref in sig.findall(".//{%s}Reference" % NS['ds']):
-        object = None
+        obj = None
+        verified = None
         uri = ref.get('URI', None)
         if uri is None or uri == '#' or uri == '':
             ct = _remove_child_comments(_root(copy.deepcopy(t)))
-            object = _root(ct)
+            obj = _root(ct)
         elif uri.startswith('#'):
             ct = copy.deepcopy(t)
-            object = _remove_child_comments(_get_by_id(ct, uri[1:]))
+            obj = _remove_child_comments(_get_by_id(ct, uri[1:]))
         else:
             raise XMLSigException("Unknown reference %s" % uri)
 
-        if object is None:
+        if obj is None:
             raise XMLSigException("Unable to dereference Reference URI='%s'" % uri)
 
+        if return_verified:
+            verified = copy.deepcopy(obj)
+
         for tr in ref.findall(".//{%s}Transform" % NS['ds']):
             logging.debug("transform: %s" % _alg(tr))
-            object = _transform(_alg(tr), object, tr)
+            obj = _transform(_alg(tr), obj, tr)
 
         if _DEBUG_WRITE_TO_FILES:
             with open("/tmp/foo-obj.xml", "w") as fd:
-                fd.write(object)
+                fd.write(obj)
 
         dm = ref.find(".//{%s}DigestMethod" % NS['ds'])
         if dm is None:
@@ -316,14 +322,18 @@ def _process_references(t, sig=None):
         logging.debug("using hash algorithm %s" % this_hash_alg)
         hash_alg = hash_alg or this_hash_alg
         if this_hash_alg != hash_alg:
-            raise XMLSigException("Unable to handle more than one hash algorithm (%s != %s)" \
+            raise XMLSigException("Unable to handle more than one hash algorithm (%s != %s)"
                                   % (this_hash_alg, hash_alg))
-        digest = _digest(object, this_hash_alg)
+        digest = _digest(obj, this_hash_alg)
         logging.debug("digest for %s: %s" % (uri, digest))
         dv = ref.find(".//{%s}DigestValue" % NS['ds'])
         logging.debug(etree.tostring(dv))
         dv.text = digest
-    return hash_alg
+
+    if return_verified:
+        return hash_alg, verified
+    else:
+        return hash_alg
 
 ##
 # Removes HTML or XML character references and entities from a text string.
@@ -331,6 +341,7 @@ def _process_references(t, sig=None):
 # @param text The HTML (or XML) source text.
 # @return The plain text, as a Unicode string, if necessary.
 
+
 def _unescape(text):
     def fixup(m):
         text = m.group(0)
@@ -350,7 +361,7 @@ def _unescape(text):
                     text = unichr(htmlentitydefs.name2codepoint[text[1:-1]])
             except KeyError:
                 pass
-        return text # leave as is
+        return text  # leave as is
 
     return re.sub("&#?\w+;", fixup, text)
 
@@ -467,7 +478,7 @@ def b642cert(data):
     return rsa_x509_pem.parse(b642pem(data))
 
 
-def verify(t, keyspec):
+def _verify(t, keyspec):
     """
     Verify the signature(s) in an XML document.
 
@@ -484,8 +495,7 @@ def verify(t, keyspec):
     # Load and parse certificate, unless keyspec is a fingerprint.
     cert = _load_keyspec(keyspec)
 
-    validated = False
-
+    validated = []
     for sig in t.findall(".//{%s}Signature" % NS['ds']):
         sv = sig.findtext(".//{%s}SignatureValue" % NS['ds'])
         if sv is None:
@@ -511,7 +521,7 @@ def verify(t, keyspec):
         if this_cert is None:
             raise XMLSigException("Could not find certificate to validate signature")
 
-        hash_alg = _process_references(t, sig)
+        hash_alg, ref = _process_references(t, sig)
         si = sig.find(".//{%s}SignedInfo" % NS['ds'])
         b_digest = _create_signature_digest(si, hash_alg)
 
@@ -520,11 +530,19 @@ def verify(t, keyspec):
 
         if expected != actual:
             raise XMLSigException("Signature validation failed")
-        validated = True
+        validated.append(ref)
 
     return validated
 
 
+def verify(t, keyspec):
+    return len(_verify(t, keyspec)) > 0
+
+
+def verified(t, keyspec):
+    return _verify(t, keyspec)
+
+
 ## TODO - support transforms with arguments
 def _signed_info_transforms(transforms):
     ts = [DS.Transform(Algorithm=t) for t in transforms]
@@ -560,32 +578,31 @@ def sign(t, key_spec, cert_spec=None, reference_uri=''):
 
     :param t: XML as lxml.etree
     :param key_spec: private key reference, see _load_keyspec() for syntax.
-    :param cert_spec: None or public key reference, see _load_keyspec() for syntax.
-        None is only valid if key_spec is a pkcs11:// URL
+    :param cert_spec: None or public key reference (to add cert to document), see _load_keyspec() for syntax.
     :param reference_uri: Envelope signature reference URI
     :returns: XML as lxml.etree (for convenience, 't' is modified in-place)
     """
-    do_padding = False # only in the case of our fallback keytype do we need to do pkcs1 padding here
+    do_padding = False  # only in the case of our fallback keytype do we need to do pkcs1 padding here
 
     private = _load_keyspec(key_spec, private=True)
     if private is None:
         raise XMLSigException("Unable to load private key from '%s'" % key_spec)
 
     if private['source'] == 'file':
-        do_padding = True # need to do p1 padding in this case
+        do_padding = True  # need to do p1 padding in this case
 
     if cert_spec is None and private['source'] == 'pkcs11':
         cert_spec = private['data']
-        logging.debug("Using P11 cert_spec :\n%s" % (cert_spec))
+        logging.debug("Using P11 cert_spec :\n%s" % cert_spec)
 
     public = _load_keyspec(cert_spec)
     if public is None:
-        raise XMLSigException("Unable to load public key from '%s'" % cert_spec)
-
-    logging.debug("Using %s/%s bit key" % (public['keysize'], private['keysize']))
-
-    #if private['source'] == 'pkcs11':
-    #    assert 0
+        raise XMLSigException("Unable to load public key from '%s'" \
+                              % (cert_spec))
+    if public['keysize'] != private['keysize']:
+        raise XMLSigException("Public and private key sizes do not match (%s, %s)" \
+                                  % (public['keysize'], private['keysize']))
+    logging.debug("Using %s bit key" % (private['keysize']))
 
     if t.find(".//{%s}Signature" % NS['ds']) is None:
         add_enveloped_signature(t, reference_uri=reference_uri)
@@ -595,20 +612,21 @@ def sign(t, key_spec, cert_spec=None, reference_uri=''):
             fd.write(etree.tostring(_root(t)))
 
     for sig in t.findall(".//{%s}Signature" % NS['ds']):
-        hash_alg = _process_references(t, sig)
+        hash_alg = _process_references(t, sig, return_verified=False)
         si = sig.find(".//{%s}SignedInfo" % NS['ds'])
         b_digest = _create_signature_digest(si, hash_alg)
 
         # RSA sign hash digest and insert it into the XML
-        tbs = _signed_value(b_digest, public['keysize'], do_padding, hash_alg)
+        tbs = _signed_value(b_digest, private['keysize'], do_padding, hash_alg)
         signed = private['f_private'](tbs)
         signature = b64e(signed)
         logging.debug("SignatureValue: %s" % signature)
         si.addnext(DS.SignatureValue(signature))
 
-        # Insert cert_data as b64-encoded X.509 certificate into XML document
-        sv_elt = si.getnext()
-        sv_elt.addnext(DS.KeyInfo(DS.X509Data(DS.X509Certificate(pem2b64(public['data'])))))
+        if public is not None:
+            # Insert cert_data as b64-encoded X.509 certificate into XML document
+            sv_elt = si.getnext()
+            sv_elt.addnext(DS.KeyInfo(DS.X509Data(DS.X509Certificate(pem2b64(public['data'])))))
 
     return t
 
@@ -628,7 +646,7 @@ def _create_signature_digest(si, hash_alg):
     return b64d(digest)
 
 
-def parse_xml(data, remove_whitespace=True):
+def parse_xml(data, remove_whitespace=True, remove_comments=True):
     """
     Parse XML data into an lxml.etree and remove whitespace in the process.
 
@@ -636,5 +654,5 @@ def parse_xml(data, remove_whitespace=True):
     :param remove_whitespace: boolean
     :returns: XML as lxml.etree
     """
-    parser = etree.XMLParser(remove_blank_text=remove_whitespace)
+    parser = etree.XMLParser(remove_blank_text=remove_whitespace, remove_comments=remove_comments)
     return etree.XML(data, parser)
