@@ -3,8 +3,10 @@ from ...type import univ
 from ..ber import decoder
 from ... import error
 
+
 class BooleanDecoder(decoder.AbstractDecoder):
     protoComponent = univ.Boolean(0)
+
     def valueDecoder(self, substrate, asn1Spec, tagSet, length,
                      state, decodeFun):
         if not substrate:
@@ -16,13 +18,17 @@ class BooleanDecoder(decoder.AbstractDecoder):
             value = 0
         return self._createComponent(
             tagSet, asn1Spec
-            ).clone(value), substrate[1:]
+        ).clone(value), substrate[1:]
+
 
 codecMap = decoder.codecMap.copy()
 codecMap.update({
     univ.Boolean.tagSet: BooleanDecoder(),
-    })
+})
+
+
+class Decoder(decoder.Decoder):
+    pass
 
-class Decoder(decoder.Decoder): pass
 
 decode = Decoder(codecMap)
